package com.vitorflores.connectfit.Adapter;

import android.content.Context;
import android.content.Intent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.ContractsActivity;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Client.ProfessionalProfileActivity;
import com.vitorflores.connectfit.MessageActivity;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.Professional.ClientProfileActivity;
import com.vitorflores.connectfit.R;

import java.util.List;

// Adapter to show users on a list
public class UserAdapter extends RecyclerView.Adapter<UserAdapter.ViewHolder> {

    private Context mContext;
    private User currentUser;
    private List<User> mUsers;

    public UserAdapter(Context mContext, List<User> mUsers) {
        this.mContext = mContext;
        this.mUsers = mUsers;
    }


    // Show a user in the list
    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(mContext).inflate(R.layout.item_user, parent, false);
        return new UserAdapter.ViewHolder(view);
    }

    // Set the profile image and username of the profile
    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        final User user = mUsers.get(position);

        holder.username.setText(user.getUserName());

        if (user.getImageURL().equals("default")) {
            holder.profile_image.setImageResource(R.mipmap.ic_launcher);
        } else {
            Glide.with(mContext).load(user.getImageURL()).into(holder.profile_image);
        }

        // If you click on a single item, redirect to the correct profile
        holder.itemView.setOnClickListener(view -> {
            FirebaseUser firebaseUser = FirebaseAuth.getInstance().getCurrentUser();
            DatabaseReference reference = FirebaseDatabase.getInstance().getReference("Users").child(firebaseUser.getUid());
            //Intent intent;

            reference.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                    Intent intent;
                    currentUser = dataSnapshot.getValue(User.class);

                    assert currentUser != null;
                    if (mContext instanceof AvailableProfessionals
                            || (currentUser.getAccountType() != null
                            && !currentUser.getAccountType().equals("pro")
                            && mContext instanceof ContractsActivity)) {
                        intent = new Intent(mContext, ProfessionalProfileActivity.class);
                    } else if (currentUser.getAccountType() != null
                            && currentUser.getAccountType().equals("pro")
                            && mContext instanceof ContractsActivity) {
                        intent = new Intent(mContext, ClientProfileActivity.class);
                    } else {
                        intent = new Intent(mContext, MessageActivity.class);
                    }

                    intent.putExtra("id", user.getId());
                    mContext.startActivity(intent);
                }

                @Override
                public void onCancelled(@NonNull DatabaseError databaseError) {}
            });


        });
    }

    @Override
    public int getItemCount() {
        return mUsers.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        public TextView username;
        public ImageView profile_image;

        public ViewHolder(View itemView) {
            super(itemView);

            username = itemView.findViewById(R.id.textProfileName);
            profile_image = itemView.findViewById(R.id.imgProfileAvatar);
        }
    }
}
