package com.vitorflores.connectfit.Client;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.app.ProgressDialog;
import android.content.ContentResolver;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.webkit.MimeTypeMap;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.bumptech.glide.Glide;
import com.google.android.gms.tasks.Continuation;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.storage.FirebaseStorage;
import com.google.firebase.storage.StorageReference;
import com.google.firebase.storage.StorageTask;
import com.google.firebase.storage.UploadTask;
import com.vitorflores.connectfit.ChatsActivity;
import com.vitorflores.connectfit.ContractsActivity;
import com.vitorflores.connectfit.ExercisesActivity;
import com.vitorflores.connectfit.LoginUserActivity;
import com.vitorflores.connectfit.MainActivity;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.Professional.ClientProfileActivity;
import com.vitorflores.connectfit.R;

import java.util.HashMap;

import de.hdodenhof.circleimageview.CircleImageView;

public class AccountClientSettingsActivity extends AppCompatActivity {

    // Visual elements
    private CircleImageView mImgProfileAvatarView;
    private TextView mTextHeight, mTextWeight, mTextCity, mTextState;
    private Button mBtnSave;

    // Firebase Authentication
    private FirebaseUser mFirebaseUser;
    private DatabaseReference mUsersReference;

    // Image upload
    StorageReference storageReference;
    private static final int IMAGE_REQUEST = 1;
    private Uri mImageUri;
    private StorageTask mUploadTask;

    User user;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_account_client_settings);

        // Toolbar
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.bar_title_profile_settings);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Visual elements
        mImgProfileAvatarView = findViewById(R.id.imgProfileAvatar);
        mTextHeight = findViewById(R.id.fieldHeight);
        mTextWeight = findViewById(R.id.fieldWeight);
        mTextCity = findViewById(R.id.fieldCity);
        mTextState = findViewById(R.id.fieldState);
        mBtnSave = findViewById(R.id.btnSave);

        // Firebase Auth
        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();
        mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());

        // Image upload
        storageReference = FirebaseStorage.getInstance().getReference("uploads");

        // Fill the fields with the current values
        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                user = dataSnapshot.getValue(User.class);

                assert user != null;
                if (user.getImageURL() != null && user.getImageURL().equals("default")) {
                    mImgProfileAvatarView.setImageResource(R.mipmap.ic_launcher_round);
                } else {
                    Glide.with(getApplicationContext()).load(user.getImageURL()).into(mImgProfileAvatarView);
                }

                mTextHeight.setText(user.getHeight());
                mTextWeight.setText(user.getWeight());
                mTextCity.setText(user.getCity());
                mTextState.setText(user.getState());
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });

        // Change the profile image
        mImgProfileAvatarView.setOnClickListener(view -> openImage());

        // Update the account details
        mBtnSave.setOnClickListener(v -> {
            final String height = mTextHeight.getText().toString();
            final String weight = mTextWeight.getText().toString();
            final String city = mTextCity.getText().toString();
            final String state = mTextState.getText().toString();

            mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());

            // Combine the field values into one hashmap
            HashMap<String, java.lang.Object> hashMap = new HashMap<>();
            hashMap.put("height", height);
            hashMap.put("weight", weight);
            hashMap.put("city", city);
            hashMap.put("state", state);

            // Update the account details and redirects to the main screen
            mUsersReference.updateChildren(hashMap).addOnCompleteListener(task -> {
                if (task.isSuccessful()) {
                    Intent intent = new Intent(AccountClientSettingsActivity.this, MainActivity.class);
                    intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
                    startActivity(intent);
                    finish();
                } else {
                    Toast.makeText(AccountClientSettingsActivity.this, "Error", Toast.LENGTH_SHORT).show();
                }
            });
        });
    }

    // Open the screen to choose a new image from the gallery
    private void openImage() {
        Intent intent = new Intent();
        intent.setType("image/*");
        intent.setAction(Intent.ACTION_GET_CONTENT);
        startActivityForResult(intent, IMAGE_REQUEST);
    }

    // Get the extension of a file
    private String getFileExtension(Uri uri) {
        ContentResolver contentResolver = getApplicationContext().getContentResolver();
        MimeTypeMap mimeTypeMap = MimeTypeMap.getSingleton();

        return mimeTypeMap.getExtensionFromMimeType(contentResolver.getType(uri));
    }

    // Upload the new image to the firebase server
    private void uploadImage() {
        // Progress dialog while the image is uploading
        final ProgressDialog pd = new ProgressDialog(AccountClientSettingsActivity.this);
        pd.setMessage("Uploading");
        pd.show();

        // Upload the image
        if (mImageUri != null) {
            final StorageReference fileReference = storageReference.child(System.currentTimeMillis()
                    + "." + getFileExtension(mImageUri));

            mUploadTask = fileReference.putFile(mImageUri);
            // If the app failed to upload, show the error message
            // If the image was successful uploaded, update the profile picture with the new image
            mUploadTask.continueWithTask((Continuation<UploadTask.TaskSnapshot, Task<Uri>>) task -> {
                if (!task.isSuccessful()) {
                    throw task.getException();
                }

                return fileReference.getDownloadUrl();
            }).addOnCompleteListener(task -> {
                // If the image was successful uploaded, update the profile picture with the new image
                if (task.isSuccessful()) {
                    String uri = task.getResult().toString();

                    mUsersReference = FirebaseDatabase.getInstance().getReference("Users").child(mFirebaseUser.getUid());
                    HashMap<String, java.lang.Object> map = new HashMap<>();
                    map.put("imageURL", uri);
                    mUsersReference.updateChildren(map);
                } else {
                    Toast.makeText(AccountClientSettingsActivity.this, "Error", Toast.LENGTH_SHORT).show();
                }

                pd.dismiss();
            }).addOnFailureListener(e -> {
                // If the app failed to upload, show the error message
                Toast.makeText(AccountClientSettingsActivity.this, e.getMessage(), Toast.LENGTH_SHORT).show();
                pd.dismiss();
            });
        } else {
            Toast.makeText(AccountClientSettingsActivity.this, "No image selected", Toast.LENGTH_SHORT).show();
        }
    }

    // Wait for the user to choose the new image, after that, call the method to upload it to the server
    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == IMAGE_REQUEST && resultCode == RESULT_OK
            && data != null && data.getData() != null) {
            mImageUri = data.getData();

            if (mUploadTask != null && mUploadTask.isInProgress()) {
                Toast.makeText(AccountClientSettingsActivity.this, "Upload in progress", Toast.LENGTH_SHORT).show();
            } else {
                uploadImage();
            }
        }
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(AccountClientSettingsActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(AccountClientSettingsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(AccountClientSettingsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(AccountClientSettingsActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(AccountClientSettingsActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(AccountClientSettingsActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(AccountClientSettingsActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}