package com.vitorflores.connectfit.Client;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Adapter.UserAdapter;
import com.vitorflores.connectfit.ChatsActivity;
import com.vitorflores.connectfit.ContractsActivity;
import com.vitorflores.connectfit.ExercisesActivity;
import com.vitorflores.connectfit.LocationSearchActivity;
import com.vitorflores.connectfit.LoginUserActivity;
import com.vitorflores.connectfit.MainActivity;
import com.vitorflores.connectfit.Model.User;
import com.vitorflores.connectfit.R;

import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

public class AvailableProfessionals extends AppCompatActivity {

    // Visual elements
    FloatingActionButton mBtnLocationSearch;
    EditText mFieldSearch;
    private RecyclerView listAvailableProfessionals;
    private UserAdapter userAdapter;

    private List<User> mProfessionals;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_available_professionals);

        // Toolbar settings
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.bar_title_available_professionals);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Visual elements
        mBtnLocationSearch = findViewById(R.id.btnLocationSearch);

        // Go to the maps activity, to find a personal trainer near you
        mBtnLocationSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent intent = new Intent(AvailableProfessionals.this, LocationSearchActivity.class);
                startActivity(intent);
                //finish();
            }
        });

        // Watch for characters added on the search field
        mFieldSearch = findViewById(R.id.fieldSearch);
        mFieldSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
                String searchTerm = StringUtils.stripAccents(charSequence.toString().toLowerCase());
                searchUsers(searchTerm);
            }

            @Override
            public void afterTextChanged(Editable editable) {
                /*Intent intent = new Intent(AvailableProfessionals.this, LocationSearchActivity.class);
                startActivity(intent);
                //finish();*/
            }
        });

        // Show the list of available professionals
        listAvailableProfessionals = findViewById(R.id.listAvailableProfessionals);
        //recyclerView.setHasFixedSize(true);
        listAvailableProfessionals.setLayoutManager(new LinearLayoutManager(AvailableProfessionals.this));

        mProfessionals = new ArrayList<>();
        readProfessionals();
    }

    private void searchUsers(String searchTerm) {
        final FirebaseUser firebaseUser = FirebaseAuth.getInstance().getCurrentUser();
        // Search for usernames starting with the text on the search field
        Query query = FirebaseDatabase.getInstance().getReference("Users").orderByChild("searchableName")
                .startAt(searchTerm)
                .endAt(searchTerm + "\uf8ff");

        query.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                // Show the search results
                mProfessionals.clear();

                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    User user = snapshot.getValue(User.class);

                    assert user != null;
                    assert firebaseUser != null;
                    if (user.getId() != null && !user.getId().equals(firebaseUser.getUid()) && user.getAccountType().equals("pro")) {
                        mProfessionals.add(user);
                    }
                }

                userAdapter = new UserAdapter(AvailableProfessionals.this, mProfessionals);
                listAvailableProfessionals.setAdapter(userAdapter);

            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });
    }


    // Make a list of the available professionals and show it to the user
    private void readProfessionals() {
        final FirebaseUser firebaseUser = FirebaseAuth.getInstance().getCurrentUser();
        DatabaseReference reference = FirebaseDatabase.getInstance().getReference("Users");

        reference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                // Show all the professionals if the searchbar is empty
                if (mFieldSearch.getText().toString().equals("")) {
                    mProfessionals.clear();

                    // Loop through the "pro" users
                    for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                        User user = snapshot.getValue(User.class);

                        assert user != null;
                        assert firebaseUser != null;
                        if (user.getId() != null && !user.getId().equals(firebaseUser.getUid()) && user.getAccountType().equals("pro")) {
                            mProfessionals.add(user);
                        }
                    }

                    // Take the professionals list and use an adapter to show on the screen
                    userAdapter = new UserAdapter(AvailableProfessionals.this, mProfessionals);
                    listAvailableProfessionals.setAdapter(userAdapter);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {

            }
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(AvailableProfessionals.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(AvailableProfessionals.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(AvailableProfessionals.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(AvailableProfessionals.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(AvailableProfessionals.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(AvailableProfessionals.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(AvailableProfessionals.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}