package com.vitorflores.connectfit;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.RelativeLayout;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Adapter.UserAdapter;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Model.Contract;
import com.vitorflores.connectfit.Model.User;

import java.util.ArrayList;
import java.util.List;

public class ContractsActivity extends AppCompatActivity {

    // Visual elements
    RelativeLayout mContractsContainer, mPendingContractsContainer, mActiveContractsContainer;
    RecyclerView mContractsLayout, mPendingContractsLayout, mActiveContractsLayout;

    // Firebase Authentication
    FirebaseUser mFirebaseUser;
    DatabaseReference mUsersReference, mContractsReference;

    private UserAdapter userAdapter, pendingUserAdapter, activeUserAdapter;
    private List<User> mUsers, mPendingUsers, mActiveUsers;
    private List<Contract> usersList;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_contracts);

        Intent intent = getIntent();
        final String typeContracts = intent.getStringExtra("typeContracts");

        // Toolbar
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        if (typeContracts.equals("active")) {
            getSupportActionBar().setTitle(R.string.section_title_active);
        } else {
            getSupportActionBar().setTitle(R.string.section_title_pending);
        }

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Visual elements
        mContractsContainer = findViewById(R.id.contractsContainer);
        mContractsLayout = findViewById(R.id.contractsLayout);
        mContractsLayout.setHasFixedSize(true);
        mContractsLayout.setLayoutManager(new LinearLayoutManager(ContractsActivity.this));

        // Firebase Authentication
        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();

        usersList = new ArrayList<>();

        // If you have a contract (pending or active), add his userid to a list
        mContractsReference = FirebaseDatabase.getInstance().getReference("Contracts").child(mFirebaseUser.getUid());
        mContractsReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                usersList.clear();

                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    Contract contract = snapshot.getValue(Contract.class);

                    assert contract != null;
                    if (contract.getStatus() != null && contract.getStatus().equals(typeContracts)) {
                        usersList.add(contract);
                    }
                }

                // Take the list of userids and show their names on the screen
                showContracts();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {}
        });
    }

    private void showContracts() {
        mUsers = new ArrayList<>();

        // Loop through the users and select the ones who have their id on your contracts list
        // After that, add the user and his info to a list
        mUsersReference = FirebaseDatabase.getInstance().getReference("Users");
        mUsersReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                mUsers.clear();

                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    User user = snapshot.getValue(User.class);

                    for (Contract contract : usersList) {
                        assert user != null;
                        assert contract != null;
                        if (user.getId() != null && contract.getId() != null && user.getId().equals(contract.getId())) {
                            mUsers.add(user);
                        }
                    }
                }

                // Take this list of users and their info and show them on the screen
                if (!mUsers.isEmpty()) {
                    userAdapter = new UserAdapter(ContractsActivity.this, mUsers);
                    mContractsLayout.setAdapter(userAdapter);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {}
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(ContractsActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(ContractsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(ContractsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(ContractsActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(ContractsActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(ContractsActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(ContractsActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}