package com.vitorflores.connectfit;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Switch;
import android.widget.Toast;

import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Model.Exercise;
import com.vitorflores.connectfit.Model.Workout;

import java.util.HashMap;

public class EditWorkoutActivity extends AppCompatActivity {

    // Visual elements
    private EditText mFieldName, mFieldDate, mFieldWorkout;
    private Switch mFieldFinished;
    private Button mBtnSave;

    // Firebase Auth
    FirebaseUser mFirebaseUser;
    DatabaseReference mReference;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit_workout);
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        Intent intent = getIntent();
        final String workoutId = intent.getStringExtra("workoutId");
        final String proId = intent.getStringExtra("proId");
        final String clientId = intent.getStringExtra("clientId");

        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        if (workoutId != null) {
            getSupportActionBar().setTitle(R.string.bar_title_edit_workout);
        } else {
            getSupportActionBar().setTitle(R.string.bar_title_add_workout);
        }

        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Visual elements
        mFieldName = findViewById(R.id.fieldName);
        mFieldFinished = findViewById(R.id.switchFinished);
        mFieldDate = findViewById(R.id.fieldDate);
        mFieldWorkout = findViewById(R.id.fieldWorkout);
        mBtnSave = findViewById(R.id.btnSave);

        // If edit workout screen
        if (workoutId != null && clientId != null) {
            mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();
            if (accountType.equals("pro")) {
                mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(mFirebaseUser.getUid()).child(clientId).child(workoutId);
            } else {
                mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(proId).child(mFirebaseUser.getUid()).child(workoutId);
            }

            mReference.addValueEventListener(new ValueEventListener() {
                @Override
                public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                    Workout workout = dataSnapshot.getValue(Workout.class);

                    assert workout != null;
                    if (workout != null) {

                        mFieldName.setText(workout.getName());
                        if (!workout.getIsPending()) {
                            mFieldFinished.setChecked(true);
                        }
                        mFieldDate.setText(workout.getDate());
                        mFieldWorkout.setText(workout.getWorkout());
                    }
                }

                @Override
                public void onCancelled(@NonNull DatabaseError databaseError) {

                }
            });
        }

        mBtnSave.setOnClickListener(view -> {
            final String name = mFieldName.getText().toString();

            if (!name.equals("")) {
                if (workoutId != null) {
                    saveWorkout(true, proId, clientId, workoutId);
                } else {
                    saveWorkout(false, proId, clientId, workoutId);
                }
            } else {
                Toast.makeText(EditWorkoutActivity.this, "Erro ao salvar treino", Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void saveWorkout(boolean editMode, String proId, String clientId, String workoutId) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);
        final String name = mFieldName.getText().toString();
        final boolean isFinishedChecked = mFieldFinished.isChecked();
        final String date = mFieldDate.getText().toString();
        final String workout = mFieldWorkout.getText().toString();
        FirebaseUser firebaseUser = FirebaseAuth.getInstance().getCurrentUser();


        assert firebaseUser != null;
        HashMap<String, Object> hashMap = new HashMap<>();
        if (accountType.equals("pro")) {
            hashMap.put("proId", firebaseUser.getUid());
        } else {
            hashMap.put("proId", proId);
        }

        hashMap.put("clientId", clientId);
        hashMap.put("name", name);
        if (isFinishedChecked) {
            hashMap.put("isPending", false);
        } else {
            hashMap.put("isPending", true);
        }
        hashMap.put("date", date);
        hashMap.put("workout", workout);

        if (editMode) {
            if (accountType.equals("pro")) {
                mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(firebaseUser.getUid()).child(clientId).child(workoutId);
            } else {
                mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(proId).child(firebaseUser.getUid()).child(workoutId);
            }
            mReference.updateChildren(hashMap).addOnCompleteListener(task1 -> {
                if (task1.isSuccessful()) {
                    Toast.makeText(EditWorkoutActivity.this, "Treino salvo com sucesso", Toast.LENGTH_SHORT).show();
                    Intent intent = new Intent(EditWorkoutActivity.this, WorkoutsActivity.class);
                    intent.putExtra("proId", proId);
                    intent.putExtra("clientId", clientId);
                    startActivity(intent);
                    finish();
                } else {
                    Toast.makeText(EditWorkoutActivity.this, "Erro ao salvar treino", Toast.LENGTH_SHORT).show();
                }
            });
        } else {
            if (accountType.equals("pro")) {
                mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(firebaseUser.getUid()).child(clientId);
            } else {
                mReference = FirebaseDatabase.getInstance().getReference("Workouts").child(proId).child(firebaseUser.getUid());
            }
            final String id = mReference.push().getKey();
            hashMap.put("id", id);
            mReference.child(id).setValue(hashMap).addOnCompleteListener(task1 -> {
                if (task1.isSuccessful()) {
                    Toast.makeText(EditWorkoutActivity.this, "Treino salvo com sucesso", Toast.LENGTH_SHORT).show();
                    Intent intent = new Intent(EditWorkoutActivity.this, WorkoutsActivity.class);
                    intent.putExtra("proId", proId);
                    intent.putExtra("clientId", clientId);
                    startActivity(intent);
                    finish();
                } else {
                    Toast.makeText(EditWorkoutActivity.this, "Erro ao salvar treino", Toast.LENGTH_SHORT).show();
                }
            });
        }
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(EditWorkoutActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(EditWorkoutActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(EditWorkoutActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(EditWorkoutActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(EditWorkoutActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(EditWorkoutActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(EditWorkoutActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}