package com.vitorflores.connectfit;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.RelativeLayout;

import com.google.android.material.floatingactionbutton.FloatingActionButton;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.auth.FirebaseUser;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.vitorflores.connectfit.Adapter.ExerciseAdapter;
import com.vitorflores.connectfit.Adapter.WorkoutAdapter;
import com.vitorflores.connectfit.Client.AvailableProfessionals;
import com.vitorflores.connectfit.Model.Exercise;
import com.vitorflores.connectfit.Model.Workout;

import java.util.ArrayList;
import java.util.List;

public class WorkoutsActivity extends AppCompatActivity {

    // Visual elements
    RelativeLayout mWorkoutsContainer;
    RecyclerView mWorkoutsLayout;
    FloatingActionButton mBtnAddWorkout;

    FirebaseUser mFirebaseUser;
    DatabaseReference mWorkoutsReference;

    private List<Workout> workoutsList;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_workouts);

        Intent prevIntent = getIntent();
        final String proId = prevIntent.getStringExtra("proId");
        final String clientId = prevIntent.getStringExtra("clientId");
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        // Toolbar settings
        Toolbar toolbar = findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setTitle(R.string.bar_title_workouts);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        toolbar.setNavigationOnClickListener(view -> finish());

        // Visual elements
        mWorkoutsContainer = findViewById(R.id.workoutsContainer);
        mWorkoutsLayout = findViewById(R.id.workoutsLayout);
        mWorkoutsLayout.setHasFixedSize(true);
        mWorkoutsLayout.setLayoutManager(new LinearLayoutManager(WorkoutsActivity.this));
        mBtnAddWorkout = findViewById(R.id.btnAddWorkout);

        // Firebase Authentication
        mFirebaseUser = FirebaseAuth.getInstance().getCurrentUser();

        if (accountType.equals("pro")) {
            mBtnAddWorkout.setVisibility(View.VISIBLE);
        }

        mBtnAddWorkout.setOnClickListener(view -> {
            Intent intent = new Intent(WorkoutsActivity.this, EditWorkoutActivity.class);
            intent.putExtra("clientId", clientId);
            startActivity(intent);
        });

        workoutsList = new ArrayList<>();

        // If you have workouts, add its id to a list
        assert mFirebaseUser != null;
        assert proId != null;
        assert clientId != null;
        if (accountType.equals("pro")) {
            mWorkoutsReference = FirebaseDatabase.getInstance().getReference("Workouts").child(mFirebaseUser.getUid()).child(clientId);
        } else {
            mWorkoutsReference = FirebaseDatabase.getInstance().getReference("Workouts").child(proId).child(mFirebaseUser.getUid());
        }

        mWorkoutsReference.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                workoutsList.clear();

                for (DataSnapshot snapshot : dataSnapshot.getChildren()) {
                    Workout workout = snapshot.getValue(Workout.class);

                    assert workout != null;
                    workoutsList.add(workout);
                }

                // Take the list of ids and show its names on the screen
                if (!workoutsList.isEmpty()) {
                    WorkoutAdapter workoutAdapter = new WorkoutAdapter(WorkoutsActivity.this, workoutsList);
                    mWorkoutsLayout.setAdapter(workoutAdapter);
                }
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {}
        });
    }

    // Create the options menu on the toolbar
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        SharedPreferences accountPrefs = getSharedPreferences("accountPrefs", 0);
        final String accountType = accountPrefs.getString("accountType", null);

        if (accountType.equals("pro")) {
            getMenuInflater().inflate(R.menu.menu_pro, menu);
        } else {
            getMenuInflater().inflate(R.menu.menu_client, menu);
        }

        return true;
    }

    // If an option on menu is clicked
    @Override
    public boolean onOptionsItemSelected(@NonNull MenuItem item) {
        Intent intent;
        switch (item.getItemId()) {
            // Main Screen
            case R.id.mainScreen:
                startActivity(new Intent(WorkoutsActivity.this, MainActivity.class));
                finish();
                return true;
            // Active contracts
            case R.id.activeContracts:
                intent = new Intent(WorkoutsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "active");
                startActivity(intent);
                finish();
                return true;
            // Pending contracts
            case R.id.pendingContracts:
                intent = new Intent(WorkoutsActivity.this, ContractsActivity.class);
                intent.putExtra("typeContracts", "pending");
                startActivity(intent);
                finish();
                return true;
            // Available Professionals Screen
            case R.id.availableProfessionals:
                startActivity(new Intent(WorkoutsActivity.this, AvailableProfessionals.class));
                finish();
                return true;
            // Exercises Screen
            case R.id.exercises:
                startActivity(new Intent(WorkoutsActivity.this, ExercisesActivity.class));
                finish();
                return true;
            // Chats Screen
            case R.id.chats:
                startActivity(new Intent(WorkoutsActivity.this, ChatsActivity.class));
                finish();
                return true;
            // Log out
            case R.id.logout:
                FirebaseAuth.getInstance().signOut();
                startActivity(new Intent(WorkoutsActivity.this, LoginUserActivity.class));
                finish();
                return true;
        }

        return false;
    }
}